@@ -183,16 +183,17 @@ KBackground::KBackground( QWidget *parent, int mode, int desktop )
   groupLayout->addSpacing( 15 );
   groupLayout->addWidget( deskListBox, 5 );
 	
-  renameButton = new QPushButton( i18n( "&Rename ..." ), group );
-  renameButton->adjustSize();
-  renameButton->setFixedHeight( renameButton->height() );
-  renameButton->setMinimumWidth( renameButton->width() );
-  renameButton->setEnabled( !oneDesktopMode );
-  if ( !KWM::isKWMInitialized() )
-    renameButton->setEnabled( false );
-  connect( renameButton, SIGNAL( clicked() ), SLOT( slotRenameDesk() ) );
+// suppress rename button (kcmpanel provides this, duncan@kde.org 19990822
+//  renameButton = new QPushButton( i18n( "&Rename ..." ), group );
+//  renameButton->adjustSize();
+//  renameButton->setFixedHeight( renameButton->height() );
+//  renameButton->setMinimumWidth( renameButton->width() );
+//  renameButton->setEnabled( !oneDesktopMode );
+//  if ( !KWM::isKWMInitialized() )
+//    renameButton->setEnabled( false );
+//  connect( renameButton, SIGNAL( clicked() ), SLOT( slotRenameDesk() ) );
 	
-  groupLayout->addWidget( renameButton, 5 );
+//  groupLayout->addWidget( renameButton, 5 );
 
   oneDesktopButton = new QCheckBox( i18n("&Common Background"), group );
   oneDesktopButton->adjustSize();
@@ -494,28 +495,16 @@ void KBackground::readSettings( int num )
   QString group;
   ksprintf( &group, "/desktop%drc", num);
 
-  bool first_time = false;
-
-  QFileInfo fi( KApplication::localconfigdir() + group );
-  if ( ! fi.exists() ){
-    first_time = true;
-    group = "/kcmdisplayrc";
-  }
-
   KConfig config(KApplication::kde_configdir() + group,
 		  KApplication::localconfigdir() + group);
 
-  if ( !first_time ) {
-    config.setGroup( "Common" );
-    randomMode = config.readBoolEntry( "RandomMode", DEFAULT_ENABLE_RANDOM_MODE);
+  config.setGroup( "Common" );
+  randomMode = config.readBoolEntry( "RandomMode", DEFAULT_ENABLE_RANDOM_MODE);
 
-    if ( randomMode || !interactive )
-      ksprintf( &group, "Desktop%d", random );
-    else
-      ksprintf( &group, "Desktop%d", DEFAULT_DESKTOP);
-  }
+  if ( randomMode || !interactive )
+    ksprintf( &group, "Desktop%d", random );
   else
-    ksprintf( &group, "Desktop%d", num + 1);
+    ksprintf( &group, "Desktop%d", DEFAULT_DESKTOP);
 
   config.setGroup( group );
   
@@ -1319,7 +1308,7 @@ void KBackground::slotToggleOneDesktop()
     rnddlg->done( QDialog::Accepted );
   
   oneDesktopButton->setChecked( oneDesktopMode );
-  renameButton->setEnabled( !oneDesktopMode );
+//  renameButton->setEnabled( !oneDesktopMode );
   deskListBox->setEnabled( !oneDesktopMode );
 
   changed = true;
@@ -1914,7 +1903,9 @@ void KRandomDlg::addToPicList( QString pic )
 
 void KRandomDlg::slotAdd()
 {
-  addToPicList( i18n("No wallpaper") );
+  QString name=KFileDialog::getOpenFileName(kapp->kde_wallpaperdir());
+  if(!name.isEmpty())
+    addToPicList(name);
 }
 
 
@@ -2020,6 +2011,7 @@ void KRandomDlg::done( int r )
   picturesConfig.writeEntry( "InOrder", orderButton->isChecked() );
   picturesConfig.writeEntry( "UseDir", dirCheckBox->isChecked() );
   picturesConfig.writeEntry( "Directory", dirLined->text() );
+  picturesConfig.writeEntry( "RandomMode", true );
 
   hide();
   
@@ -2027,6 +2019,7 @@ void KRandomDlg::done( int r )
 
   kb->rnddlg = 0L;
   kb->random = 0;
+  kb->randomMode = true;
   kb->randomSetupButton->setEnabled( true );
   kb->wpModeCombo->setEnabled( false );
 }
