@@ -18,6 +18,8 @@
 #include <stdio.h>
 #include <linux/kernel.h>
 
+#include <ctype.h>
+
 #include <kapp.h>
 #include <ktablistbox.h>
 
@@ -57,7 +59,6 @@
 
 bool GetInfo_ReadfromFile( KTabListBox *lBox, char *Name, char splitchar  )
 {
-  QString str;
   char buf[512];
 
   QFile *file = new QFile(Name);
@@ -72,7 +73,10 @@ bool GetInfo_ReadfromFile( KTabListBox *lBox, char *Name, char splitchar  )
       {  char *p=buf;
          if (splitchar!=0)    /* remove leading spaces between ':' and the following text */
 	     while (*p)
-	     {	if (*p==splitchar)
+	     {
+		if (!isgraph(*p))
+			*p = ' ';
+		if (*p==splitchar)
 	        { *p++ = ' ';
 		  while (*p==' ') ++p;
 		  *(--p) = splitchar;
@@ -80,6 +84,16 @@ bool GetInfo_ReadfromFile( KTabListBox *lBox, char *Name, char splitchar  )
 		}
 		else ++p;
 	     }
+	 else
+	 {
+		while (*p)
+		{
+		   if (!isgraph(*p))
+			*p = ' ';
+		   ++p;
+		}
+	 }
+
          lBox->setSeparator(splitchar);
          lBox->insertItem(buf);
       }
