@@ -1,7 +1,7 @@
     /*
 
     Greeter module for xdm
-    $Id: kgreeter.cpp,v 1.56.4.3 1999/04/07 00:34:49 steffen Exp $
+    $Id: kgreeter.cpp,v 1.56.4.5 1999/08/16 11:36:34 knoll Exp $
 
     Copyright (C) 1997, 1998 Steffen Hansen
                              stefh@mip.ou.dk
@@ -90,36 +90,20 @@ Display                 ** dpy;
 struct verify_info      *verify;
 struct greet_info       *greet;
 
-// Here we store a number for the seed:
-//static int event_sum;
+void 
+KLoginLineEdit::focusOutEvent( QFocusEvent *e) 
+{
+     emit lost_focus();
+     QLineEdit::focusOutEvent( e);
+}
 
-class MyApp:public KApplication {
+class MyApp : public KApplication {
 public:
      MyApp( int &argc, char **argv );
      virtual ~MyApp();
      virtual bool x11EventFilter( XEvent * );
-
-     /* Calculate a "sum" of the recorded events
-      * The "sum" is caclulated in a somewhat ad hoc
-      * way by starting with the value of a pointer
-      * and shifting and xor'ing events to it. See
-      * the implementation.
-      * 
-      * If someone has a more "proven" way to do this,
-      * please contact me /stefh
-      *
-      * NOTE: This is disabled now. We read /dev/mem instead!
-      * /stefh
-      */
-     //static int getSum() { return event_sum;}
-private:
-     // Add one timestamp
-     //void addEvent( int);
-     //static int event_sum;
 };
 
-//int MyApp::event_sum = 42;
-
 MyApp::MyApp(int &argc, char **argv ) : KApplication(argc, argv)
 {}
 
@@ -128,12 +112,6 @@ MyApp::~MyApp()
 
 bool 
 MyApp::x11EventFilter( XEvent * ev){
-#if 0 
-    if( ev->type == KeyPress ||
-	 ev->type == KeyRelease ) addEvent( ((XKeyEvent*)ev)->time);
-     if( ev->type == ButtonPress ||
-	 ev->type == ButtonRelease ) addEvent( ((XButtonEvent*)ev)->time);
-#endif
      if( ev->type == KeyPress && kgreeter){
 	  // This should go away
 	  KeySym ks = XLookupKeysym(&(ev->xkey),0);
@@ -152,20 +130,6 @@ MyApp::x11EventFilter( XEvent * ev){
      return FALSE;
 }
 
-#if 0
-void
-MyApp::addEvent( int t)
-{
-     event_sum = (event_sum << 3) ^ t;
-     printf("addEvent( %d): event_sum = %d\n", t, event_sum);
-}
-
-// GenerateAuthData calls this to get the seed:
-extern "C" {
-     int greeter_event_sum() { printf("%d\n",MyApp::getSum());return MyApp::getSum();}
-}
-#endif
-
 // Misc. functions
 static inline int my_seteuid( uid_t euid)
 {
@@ -233,7 +197,7 @@ KGreeter::KGreeter(QWidget *parent = 0, const char *t = 0)
 
      loginLabel = new QLabel( klocale->translate("Login:"), this);
      set_min( loginLabel);
-     loginEdit = new QLineEdit( this);
+     loginEdit = new KLoginLineEdit( this);
 
      // The line-edit look _very_ bad if you don't give them 
      // a resonal height that observes a proportional aspect.
@@ -366,6 +330,8 @@ KGreeter::KGreeter(QWidget *parent = 0, const char *t = 0)
      // Signals/Slots
      connect( timer, SIGNAL(timeout()),
            this , SLOT(timerDone()) );
+     connect( loginEdit, SIGNAL(lost_focus()),
+	      this, SLOT( load_wm()));
      if( user_view)
 	  connect( user_view, SIGNAL(selected(int)), 
 		   this, SLOT(slot_user_name( int)));
@@ -674,6 +640,7 @@ KGreeter::restrict_expired(){
      bool quietlog = false;
      time_t warntime = DEFAULT_WARN;
 #endif
+#ifndef __osf__
      if (pwd->pw_expire)
 	  if (pwd->pw_expire <= time(NULL)) {
 	       QMessageBox::critical(NULL, NULL,
@@ -688,6 +655,7 @@ KGreeter::restrict_expired(){
 				     str,
 				     i18n("&OK"));
 	  }
+#endif // __osf__
 
      return false;
 }
