@@ -1,6 +1,6 @@
 /* $XConsortium: session.c,v 1.72.1.1 95/06/19 20:29:12 gildea Exp $ */
 /* $XFree86: xc/programs/xdm/session.c,v 3.7 1995/07/08 10:32:08 dawes Exp $ */
-/* $Id: session.c,v 1.13 1998/09/29 16:44:21 bieker Exp $ */
+/* $Id: session.c,v 1.13.4.5 1999/08/20 18:35:10 rkrusty Exp $ */
 /*
 
 Copyright (c) 1988  X Consortium
@@ -258,7 +258,9 @@ static	struct dlfuncs	dlfuncs = {
 	endspent,
 #endif
 	getpwnam,
+#ifdef HAVE_CRYPT_H
 	crypt,
+#endif
 	};
 	
 #ifdef X_NOT_STDC_ENV
@@ -605,6 +607,14 @@ extern void ResetServer( struct display *d );
 void SessionExit (d, status, removeAuth)
     struct display  *d;
 {
+#ifdef USE_PAM
+    if( pamh) {
+        /* shutdown PAM session */
+        pam_close_session(pamh, 0);
+        pam_end(pamh, PAM_SUCCESS);
+        pamh = NULL;
+    }
+#endif
     /* make sure the server gets reset after the session is over */
     if (d->serverPid >= 2 && d->resetSignal)
 	kill (d->serverPid, d->resetSignal);
@@ -643,14 +653,6 @@ void SessionExit (d, status, removeAuth)
 	    }
 	}
 #endif /* K5AUTH */
-#ifdef USE_PAM
-	if( pamh) {
-	  /* shutdown PAM session */
-	  pam_close_session(pamh, 0);
-	  pam_end(pamh, PAM_SUCCESS);
-	  pamh = NULL;
-	}
-#endif
     }
     Debug ("Display %s exiting with status %d\n", d->name, status);
     exit (status);
@@ -697,6 +699,18 @@ StartClient (verify, d, pidp, name, passwd)
 
 	/* Do system-dependent login setup here */
 
+#ifdef USE_PAM
+    /* pass in environment variables set by libpam and modules it called */
+	if (pamh) {
+	  long i;
+	  char **pam_env = pam_getenvlist(pamh);
+	  for(i = 0; pam_env && pam_env[i]; i++)
+	    {
+	      verify->userEnviron = putEnv(pam_env[i], verify->userEnviron);
+	    }
+	}
+#endif
+
 #ifdef HAVE_SETUSERCONTEXT
         /*
          * Destroy environment unless user has requested its preservation.
@@ -1064,7 +1078,7 @@ char	*user, *home;
     if (user)
     {
 	env = setEnv (env, "USER", user);
-#if defined(SYSV) || defined(SVR4)
+#if defined(SYSV) || defined(SVR4) || defined(linux)
 	env = setEnv (env, "LOGNAME", user);
 #endif
     }
@@ -1075,7 +1089,7 @@ char	*user, *home;
     return env;
 }
 
-#if defined(Lynx) || defined(SCO) && !defined(SCO_USA)
+#if defined(Lynx) || defined(SCO) && !defined(SCO_USA) || !defined(HAVE_CRYPT_H)
 char *crypt(s1, s2)
 	char	*s1, *s2;
 {
