@@ -23,6 +23,8 @@
 #include <qdialog.h>
 #include <qregexp.h>
 
+class QLineEdit;
+
 class KFindTextDialog : public QDialog
 {
     Q_OBJECT
@@ -32,6 +34,8 @@ public:
 
     const QRegExp &regExp() const { return rExp; }
 
+    virtual void show();
+
 signals:
     /*
      * connect to me to find out when the user has pressed the find button.
@@ -46,6 +50,7 @@ public slots:
 
 protected:
     QRegExp rExp;
+    QLineEdit * edit;
 };
 
 #endif
