@@ -3,6 +3,7 @@
 #include <string.h>
 #include <unistd.h>
 #include <sys/types.h>
+#include <pwd.h>
 #include <stddef.h>
 
 #include <qpopmenu.h>
@@ -50,7 +51,7 @@ char* kfm_getrev();
 
 char* kfm_getrev()
 {
-    static char *revision = "$Revision: 1.167.2.11 $";
+    static char *revision = "$Revision: 1.167.2.21 $";
     static char *rev;
     static char *p;
 
@@ -605,12 +606,11 @@ void KfmGui::initView()
     connect( view, SIGNAL( setTitle(const char *)), this, SLOT( slotTitle(const char *)) );
     connect( view, SIGNAL( historyUpdate( bool, bool ) ), this, SLOT( slotUpdateHistory( bool, bool ) ) );
     connect( view, SIGNAL( newURL( const char * ) ), this, SLOT( slotNewURL( const char * ) ) );
+    connect( view, SIGNAL( textSelected( KHTMLView *, bool) ), this, SLOT( slotTextSelected( KHTMLView *, bool) ) );
     connect( view, SIGNAL( documentStarted( KHTMLView * ) ), 
 	     this, SLOT( slotAddWaitingWidget( KHTMLView * ) ) );
     connect( view, SIGNAL( documentDone( KHTMLView * ) ), 
 	     this, SLOT( slotRemoveWaitingWidget( KHTMLView * ) ) );
-    connect( view, SIGNAL( textSelected( KHTMLView *, bool ) ), 
-	     this, SLOT( slotTextSelected( KHTMLView *, bool ) ) );
     view->show();
     view->setFocus();
 }
@@ -724,75 +724,88 @@ void KfmGui::slotSaveCacheOff()
 }
 
 void KfmGui::slotURLEntered()
+{
+   openFilteredURL( toolbarURL->getLinedText( TOOLBAR_URL_ID ) );
+}
+
+void KfmGui::openFilteredURL ( const char * _url )
 {
     if ( view->getActiveView() )
     {
-        QString url = toolbarURL->getLinedText( TOOLBAR_URL_ID );
-
-    	/*
-         while (url.find(' ') == 0)
-    	  url.remove(0, 1);
-    	 while (url.findRev(' ') == (signed) url.length()-1)
-	      url.remove(url.length()-1, 1);
-
-         Replacement for the above statements. Instead use
-         the built-in QString function to strip any leading
-         and trailing spaces from the URL. ( Dawit A. )
-        */
-        url.stripWhiteSpace();
-
+        QString url = _url;
+        // strip any leading and trailing spaces.
+        url = url.stripWhiteSpace();
         // Exit if the user did not enter a URL
-        if ( url.data()[0] == 0 )
+        if ( url.isEmpty() )
 	        return;
-
-    	// Root directory?
-	    if ( url.data()[0] == '/' )
-    	{
-            KURL u( toolbarURL->getLinedText( TOOLBAR_URL_ID ) );
-            url = u.url().data();
+        // If url starts with // and contains no '~',
+        // clean it up so kfm can behave much like
+        // a normal shell would. (Dawit A.)
+        if ( url.find("//") == 0 && url.find ('~') == -1 )
+           url = QDir:: cleanDirPath (url.data());
+        // Strip off "file:/" in order to properly
+        // expand local URLs when necessary.
+        if ( url.find ("file:") == 0 )
+        {
+           url.remove ( 0, 5 );
+           // Home directory preceeded by "file:/" or "file://"
+           if ( url.find("/~") == 0 || url.find("//~") == 0 )
+              url.remove( 0, url.find('~') );
+           else
+              url = QDir:: cleanDirPath (url.data());
         }
         // Home directory?
-        else if ( url.data()[0] == '~' )
+        if ( url.find ( QRegExp ( "^~.*" ) ) == 0 )
         {
-            QString tmp( QDir::homeDirPath().data() );
-            tmp += toolbarURL->getLinedText( TOOLBAR_URL_ID ) + 1;
-            url = tmp.prepend("file:").data();
+            int length = url.length();
+            int index = url.find ( "/" );
+            if ( length == 1 || index == 1 )
+                url.replace ( 0, 1, QDir::homeDirPath().data() );
+            else
+            {
+              struct passwd *dir = ((index == -1) ? getpwnam(url.mid(1,length).data()) : getpwnam( url.mid(1,index-1).data()));
+               if ( !dir ) return; // unknown user
+              (index == -1) ? url.replace (0, length,  dir->pw_dir) : url.replace (0, index, dir->pw_dir);
+            }
         }
-        //Valid URL?
-        // Do nothing if URL, for our purposes here, is VALID. This check is
-        // case insensitive i.e both www & WWW are seen as valid. (Dawit A.)
-        else if ( url.find( "://" ) >= 0 ||
-                  url.find ( "mailto:", 0, false ) == 0 ||
-                  url.find( "file:/", 0, false ) == 0 ||
-                  url.find ( "news:", 0, false  ) == 0 ) ;
-
-        // No protocol. Does url begin with www? (sven)
+        // An Easter egg ? Let's have some fun first !!
+	    else if ( url.lower() == "about:kde" )
+        {
+    		url = getenv( "KDEURL" );
+		    if ( url.isEmpty() )
+			    url = "http://www.kde.org";
+        }
+        // No protocol, but begins with www? (sven)
     	else if ( url.find( "www.", 0, false ) == 0 )
-	      url.prepend("http://");  // Unecessary since we bind "http://" as default protocol.
+            url.prepend("http://");  // Is this necessary since we default to "http://" below ?? (Dawit A.)
 
-    	// No protocol. Does url begin with "ftp."?  (sven)
+    	// No protocol, but begins with "ftp."?  (sven)
 	    else if ( url.find( "ftp.", 0, false ) == 0 )
-    	  url.prepend("ftp://");
+            url.prepend("ftp://");
 
+        // Valid URL?
+        // Do nothing if url, for our purposes here, is VALID.
+        // This check is case in-sensitive . (Dawit A.)
+        else if ( url.find ( QRegExp ("[a-zA-Z0-9\\+\\.\\-]+:/?/?.*") ) == 0 ||
+                  url.find ( QRegExp ("^//*.*") ) == 0 ) ;
         /*
            When all else fails, check if the user entered a URL that
-           exists under the current directory on the local host. If
-           it is not, attach "http://" as the default protocol. This
-           provides a short URL support to users, Ex. "linux.org" (Dawit A)
+           exists under the current directory. If it does not, attach
+           "http://" as default protocol. This enables us to provide
+           short URL support to users, Ex. "linux.org" (Dawit A)
         */
         else
         {
             KURL path ( getURL() );
-            QDir::setCurrent ( path.directory () );
+            QDir::setCurrent ( path.directory()  );
             QFileInfo f ( QDir::currentDirPath().append( "/" ).append( url ).data() );
-            debug ( f.filePath() );
-        	if ( f.exists() )
+            if ( f.exists() )
                 url = f.filePath();
-            else
+            else if ( url[ url.length()-1 ] != ':' && url[0] != '/' &&
+                      url.find ('.') > 0 && url.find(' ') == -1 )
                 url = url.prepend ("http://");
         }
-
-	    KURL u( url.data() );
+    	KURL u( url.data() );
     	if ( u.isMalformed() )
 	    {
 	        QString tmp;
@@ -800,11 +813,12 @@ void KfmGui::slotURLEntered()
 	        QMessageBox::critical( (QWidget*)0L, klocale->translate( "KFM Error" ), tmp );
             return;
         }
-        view->openURL( url.data() );
+        view->openURL( u.url().data() );
+        view->getActiveView()->setFocus();
         //  update tree view Sep 5 rjakob
-        if (url.left(5)=="file:")
-    	    if (bTreeView && pkfm->isTreeViewFollowMode())
-		treeView->slotshowDirectory(url.data()+5);
+        if ( u.protocol() == "file" )
+            if (bTreeView && pkfm->isTreeViewFollowMode())
+                treeView->slotshowDirectory(url.data()+5);
     }
 }
 
@@ -813,7 +827,6 @@ void KfmGui::setToolbarURL( const char *_url )
     KURL u(_url);
     u.setPassword(""); // hide password
     QString url(u.url().copy());
-    KURL::decodeURL(url);
     toolbarURL->setLinedText( TOOLBAR_URL_ID, url.data() );
     //  update tree view Sep 5 rjakob
     if (u.isLocalFile())
@@ -1113,7 +1126,7 @@ void KfmGui::slotFile()
     url.append(KFMPaths::TrashPath());
     if( view->getURL() == url ) 
         mfile->insertItem( klocale->getAlias(ID_STRING_TRASH), 
-		       view, SLOT( slotPopupEmptyTrashBin() ), 0, FIRSTFILEMENUITEM, 0 );
+		       view, SLOT( slotEmptyTrashBin() ), 0, FIRSTFILEMENUITEM, 0 );
     else 
     {
       mfile->insertItem( klocale->translate("&New"), menuNew, FIRSTFILEMENUITEM, 0 );
@@ -1382,59 +1395,13 @@ void KfmGui::slotOpenLocation( )
 {
     QString url = "";
     if ( view->getActiveView()->getURL() )
-	url = view->getActiveView()->getURL();
+       url = view->getActiveView()->getURL();
     
-    DlgLineEntry l( i18n("Open Location:"), url.data(), 
-		    this, true );
+    DlgLineEntry l( i18n("Open Location:"), url.data(), this, true );
     int x = l.exec();
     if ( x )
     {
-	QString url = l.getText();
-	url = url.stripWhiteSpace();
-	// Exit if the user did not enter an URL
-	if ( url.data()[0] == 0 )
-	    return;
-	// Root directory?
-	if ( url.data()[0] == '/' )
-	{
-	    url = "file:";
-	    url += l.getText();
-	}
-	// Home directory?
-	else if ( url.data()[0] == '~' )
-	{
-	    url = "file:";
-	    url += QDir::homeDirPath().data();
-	    url += l.getText() + 1;
-	}
-
-	// Some kludge to add protocol specifier on
-	// well known Locations
-	if ( url.left(4) == "www." ) {
-	    url = "http://";
-	    url += l.getText();
-	}
-	if ( url.left(4) == "ftp." ) {
-	    url = "ftp://";
-	    url += l.getText();
-	}
-	/**
-	 * Something for fun :-)
-	 */
-	if ( url == "about:kde" ) {
-    		url = getenv( "KDEURL" );
-		if ( url.isEmpty() )
-			url = "http://www.kde.org";
-	}
-	
-	KURL u( url.data() );
-	if ( u.isMalformed() )
-	{
-            warning(QString(klocale->translate("ERROR: Malformed URL"))+" : %s",u.path());
-	    return;
-	}
-	
-	view->openURL( url.data() );
+        openFilteredURL ( l.getText() );
     }
 }
 
