@@ -292,6 +292,8 @@ bool KFMManager::openURL( const char *_url, bool _reload, int _xoffset, int _yof
 	    // Add old URL to history stacks
 	    view->setUpURL( newUrlWithoutRef.url() ); // ?? (David)
 	    view->pushURLToHistory();
+	    // Leave HTML flag as is
+	    // bHTML = bHTML;
 	    if ((nextXOffset != 0) || (nextYOffset != 0))
 	    {
 	    	return (view->gotoXY(nextXOffset, nextYOffset));    	     
@@ -305,10 +307,9 @@ bool KFMManager::openURL( const char *_url, bool _reload, int _xoffset, int _yof
 
     // Is page cached ?
     const char *file;
-    if ( ( file = view->getHTMLCache()->isCached( _url ) ) != 0L && 
-         !_reload && 
-         !_data
-       )
+    if ( view->getHTMLCache()->isEnabled() &&
+         ( file = view->getHTMLCache()->isCached( _url ) ) != 0L &&
+         !_reload && !_data )
     {
 	FILE* f = fopen( file, "rb" );
 	if ( f )
@@ -328,6 +329,7 @@ bool KFMManager::openURL( const char *_url, bool _reload, int _xoffset, int _yof
 
 	    // Add old URL to history stacks
 	    view->pushURLToHistory();
+	    bHTML = true;
 
 	    url = _url;
             setUpURL();
@@ -1064,7 +1066,7 @@ void KFMManager::stop()
 
 void KFMManager::slotRedirection( const char *_url )
 {
-    kdebug(KDEBUG_INFO,1201,"Setting job URL to %s",_url);
+    // kdebug(KDEBUG_INFO,1201,"Setting job URL to %s",_url);
     jobURL = _url; // store it to a special QString.
     if (KURL(_url).isLocalFile())
     {
@@ -1205,7 +1207,7 @@ void KFMManager::slotMimeType( const char *_type )
 	    KMimeType *typ = KMimeType::findByName( aType );
 	    // Have we registered this mime type in KDE ?
 	    if ( typ && typ->run( tryURL ) ){
-	        delete typestr;
+	        if (typestr) delete [] typestr;
 		return;
              }		
 	}
@@ -1219,14 +1221,14 @@ void KFMManager::slotMimeType( const char *_type )
 	  if ( bind )
 	  {
 	    bind->runBinding( tryURL );
-	    if (typestr) delete typestr;
+	    if (typestr) delete [] typestr;
 	    return;
 	  }
 
 	  QString pattern = l.getText();
 	  if ( pattern.isEmpty() )
 	  {
-	    if (typestr) delete typestr;
+	    if (typestr) delete [] typestr;
 	    return;
 	  }	
 	  
@@ -1269,7 +1271,7 @@ void KFMManager::slotMimeType( const char *_type )
 	else
 	    bBufferPage = TRUE;
     }
-    if (typestr) delete typestr;
+    if (typestr) delete [] typestr;
 }
 
 
@@ -1452,15 +1454,11 @@ void KFMManager::openPopupMenu( QStrList &_urls, const QPoint & _point, bool _cu
     {
 	int id;
 
-        /* Commented out. Left click does it. Why have it on right click menu ?. David.
-	id = popupMenu->insertItem( klocale->getAlias(ID_STRING_CD), 
-				    view, SLOT( slotPopupCd() ) );
-        */
 	id = popupMenu->insertItem( klocale->getAlias(ID_STRING_NEW_VIEW), 
-				    view, SLOT( slotPopupNewView() ) );
+				    view, SLOT( slotNewView() ) );
 	popupMenu->insertSeparator();    
 	id = popupMenu->insertItem( klocale->getAlias(ID_STRING_TRASH), 
-				    view, SLOT( slotPopupEmptyTrashBin() ) );
+				    view, SLOT( slotEmptyTrashBin() ) );
     } 
     else if ( isdir )
     {
@@ -1485,29 +1483,22 @@ void KFMManager::openPopupMenu( QStrList &_urls, const QPoint & _point, bool _cu
 	
           popupMenu->insertSeparator();  
         }
-
-        /* Commented out. Left click does it. Why have it on right click menu ?. David.
-        if (!_current_dir) {
-            id = popupMenu->insertItem( klocale->getAlias(ID_STRING_CD), view, 
-                                        SLOT( slotPopupCd() ) );
-        }
-        */
 	id = popupMenu->insertItem( klocale->getAlias(ID_STRING_NEW_VIEW), 
-				    view, SLOT( slotPopupNewView() ) );
+				    view, SLOT( slotNewView() ) );
 	popupMenu->insertSeparator();    
 	if ( KIOServer::supports( _urls, KIO_Read ) )
 	    id = popupMenu->insertItem( klocale->getAlias( ID_STRING_COPY ), 
-					view, SLOT( slotPopupCopy() ) );
+					view, SLOT( slotCopy() ) );
 	if ( KIOServer::supports( _urls, KIO_Write ) && KfmView::clipboard->count() != 0 )
 	    id = popupMenu->insertItem( klocale->getAlias( ID_STRING_PASTE ), 
-					view, SLOT( slotPopupPaste() ) );
+					view, SLOT( slotPaste() ) );
 	if ( KIOServer::supports( _urls, KIO_Move ) &&
              !KIOServer::isTrash ( viewURL.directory() ) && !_current_dir )
 	    id = popupMenu->insertItem( klocale->getAlias( ID_STRING_MOVE_TO_TRASH ),  
-					view, SLOT( slotPopupTrash() ) );
+					view, SLOT( slotTrash() ) );
 	if ( KIOServer::supports( _urls, KIO_Delete ) && !_current_dir )
 	    id = popupMenu->insertItem( klocale->getAlias( ID_STRING_DELETE ),  
-					view, SLOT( slotPopupDelete() ) );
+					view, SLOT( slotDelete() ) );
     }
     else
     {
@@ -1515,28 +1506,27 @@ void KFMManager::openPopupMenu( QStrList &_urls, const QPoint & _point, bool _cu
         if (bHttp) {
             /* Should be for http URLs (HTML pages) only ... */
             id = popupMenu->insertItem( klocale->getAlias(ID_STRING_NEW_VIEW), 
-                                        view, SLOT( slotPopupNewView() ) );
+                                        view, SLOT( slotNewView() ) );
         }
 	id = popupMenu->insertItem( klocale->getAlias(ID_STRING_OPEN_WITH), 
-				    view, SLOT( slotPopupOpenWith() ) );
+				    view, SLOT( slotOpenWith() ) );
 	popupMenu->insertSeparator();    
 	if ( KIOServer::supports( _urls, KIO_Read ) )
 	    id = popupMenu->insertItem( klocale->getAlias( ID_STRING_COPY ), 
-					view, SLOT( slotPopupCopy() ) );
+					view, SLOT( slotCopy() ) );
 	if ( KIOServer::supports( _urls, KIO_Move )  &&
              !KIOServer::isTrash ( viewURL.directory() ) && !_current_dir )
 	    id = popupMenu->insertItem( klocale->getAlias( ID_STRING_MOVE_TO_TRASH ),  
-					view, SLOT( slotPopupTrash() ) );
+					view, SLOT( slotTrash() ) );
 	if ( KIOServer::supports( _urls, KIO_Delete ) && !_current_dir )
 	    id = popupMenu->insertItem( klocale->getAlias( ID_STRING_DELETE ),  
-					view, SLOT( slotPopupDelete() ) );
+					view, SLOT( slotDelete() ) );
     }
 
     popupMenu->insertItem( klocale->getAlias(ID_STRING_ADD_TO_BOOMARKS),
-			   view, SLOT( slotPopupBookmarks() ) );
+			   view, SLOT( slotBookmarks() ) );
 
     menuNew->setPopupFiles( _urls );
-    view->setPopupFiles( _urls );
     popupFiles.copy( _urls );
 
     QStrList bindings;
@@ -1614,7 +1604,7 @@ void KFMManager::openPopupMenu( QStrList &_urls, const QPoint & _point, bool _cu
         }
         //--------------------------------------------------------------------
 	popupMenu->insertSeparator();
-        popupMenu->insertItem( klocale->getAlias(ID_STRING_PROP), view, SLOT( slotPopupProperties() ) );
+        popupMenu->insertItem( klocale->getAlias(ID_STRING_PROP), view, SLOT( slotProperties() ) );
     }
     
     // Show the menu
@@ -1653,8 +1643,8 @@ void KFMManager::dropPopupMenu( KDNDDropZone *_zone, const char *_dest, const QP
 	    else
 	    {
 		// We did not find some binding to execute
-		QMessageBox::warning( 0, klocale->translate("KFM Error"), 
-				      klocale->translate("Dont know what to do.") );
+              // QMessageBox::warning( 0, klocale->translate("KFM Error"), 
+		              // 		      klocale->translate("Dont know what to do.") );
 		return;
 	    }
 	}
@@ -1666,7 +1656,7 @@ void KFMManager::dropPopupMenu( KDNDDropZone *_zone, const char *_dest, const QP
       char *s;
       bool attachment = false;
       for ( s = _zone->getURLList().first(); s != 0L; s = _zone->getURLList().next() )
-	if (strstr (s, "file:/tmp/kmail") != 0 )
+	if (strstr (s, "file:/") != 0 && strstr(s, "tmp/kmail") != 0 )
 	  attachment = true;
       if (attachment)
       {
@@ -1699,8 +1689,8 @@ void KFMManager::dropPopupMenu( KDNDDropZone *_zone, const char *_dest, const QP
 				     this, SLOT( slotDropLink() ) );
     if ( id == -1 )
     {
-	QMessageBox::warning( 0, klocale->translate("KFM Error"),
-			       klocale->translate("Dont know what to do.") );
+      // QMessageBox::warning( 0, klocale->translate("KFM Error"),
+	      // 		       klocale->translate("Dont know what to do.") );
 	return;
     }
 
