@@ -349,7 +349,7 @@ int KProtocolFTP::ftpPasv(void)
 	       struct sockaddr sa;
 	       struct sockaddr_in in;
     } sin;
-    struct linger lng = { 0, 0 };
+    struct linger lng = { 1, 120 };
 
     pasv=1;
     if (sDatal != -1)
@@ -400,6 +400,8 @@ int KProtocolFTP::ftpPasv(void)
 	       return 0;
     }
 
+    if (setsockopt(sDatal, SOL_SOCKET, SO_KEEPALIVE, (char *) &on, (int) sizeof(on)) < 0)
+	       perror("Keepalive not allowed.");
     if (setsockopt(sDatal, SOL_SOCKET,SO_LINGER, (char *) &lng,(int) sizeof (lng)) < 0)
 	       perror("Linger mode was not allowed.");
     return 1;
@@ -460,15 +462,15 @@ int KProtocolFTP::ftpMkdir( const char *path )
     FD_ZERO(&mask);
     FD_SET(sDatal,&mask);
 
+    if (pasv == 1)
+    {
+	    return sDatal;
+    }
     if (select( sDatal + 1, &mask,NULL,NULL, 0L) == 0)
     {
 		close(sDatal);
 		return -2;
     }
-    if (pasv == 1)
-    {
-	    return sDatal;
-    }
 
     l = sizeof(addr);
     if ((sData = accept(sDatal,&addr,&l)) > 0)
