@@ -309,7 +309,7 @@ long KProtocolHTTP::Read(void *buffer, long len)
 	long bytesPerSec = bytesRead / secs;
 	QString infoStr;
 	if ( bytesPerSec < 1000 )
-	    infoStr.sprintf( "%i bytes/s", bytesPerSec );
+	    infoStr.sprintf( "%li bytes/s", bytesPerSec );
 	else
 	    infoStr.sprintf( "%.1f kb/s", (float)bytesPerSec / (float)1000 );
 	
@@ -652,9 +652,24 @@ int KProtocolHTTP::ProcessHeader()
 	    {
 		Close();
 		KURL u( url );
+                u.setReference( 0 );
+                u.setSearchPart( 0 );
+                QString p = u.httpPath();
+                if ( p.length() > 0 )
+                {
+                    int pos = p.findRev( '/' );
+                    if ( pos >= 0 )
+                        p.truncate( pos );
+                }
+                p += "/";
+                u.setPath( p );
 		KURL u2( u, buffer + 10 );
 		if (!u2.isMalformed())
 		{
+		  if (!Cookie.isEmpty())
+		  {
+	            emit cookie( url.data(), Cookie.data() );
+		  }
 		  emit redirection( u2.url() );
 //		  return Open( &u2, currentMode );
 		  bytesleft = 0;
