@@ -978,7 +978,7 @@ void KRootWidget::slotDropEvent( KDNDDropZone *_zone )
       char *s;
       bool attachment = false;
       for ( s = _zone->getURLList().first(); s != 0L; s = _zone->getURLList().next() )
-	if (strstr (s, "file:/tmp/kmail") != 0 )
+	if (strstr (s, "file:/") != 0 && strstr(s, "tmp/kmail") != 0 )
 	  attachment = true;
       if (attachment)
       {
@@ -1367,7 +1367,7 @@ QPixmap *KRootIcon::link_pixmap = 0;
 QPixmap *KRootIcon::ro_pixmap = 0;
 
 KRootIcon::KRootIcon( const char *_url, int _x, int _y ) :
-    KDNDWidget( 0L, 0L, WStyle_Customize | WStyle_Tool | WStyle_NoBorder )
+    KDNDWidget( 0L, 0L, WStyle_Customize | WStyle_NoBorder )
 {
     grid_x = -1;
     grid_y = -1;
@@ -1861,8 +1861,8 @@ printf("dropPopupMenu\n");
 	else
 	{
 	    // We did not find some binding to execute
-	    QMessageBox::warning( 0, klocale->translate("KFM Error"),
-				  klocale->translate("Dont know what to do.") );
+          // QMessageBox::warning( 0, klocale->translate("KFM Error"),
+	          // 			  klocale->translate("Dont know what to do.") );
 	    return;
 	}
     }
@@ -1886,8 +1886,8 @@ printf("dropPopupMenu\n");
 				     this, SLOT( slotDropLink() ) );
     if ( id == -1 )
     {
-	QMessageBox::warning( 0, klocale->translate("KFM Error"),
-			      klocale->translate("Dont know what to do.") );
+      //QMessageBox::warning( 0, klocale->translate("KFM Error"),
+	      //		      klocale->translate("Dont know what to do.") );
 	return;
     }
 
