@@ -21,6 +21,8 @@
 #include <stream.h>
 #include <unistd.h>
 #include <stdlib.h>
+#include <sys/types.h>
+#include <sys/wait.h>
 #include <qfileinf.h>
 #include <qdir.h>
 #include <qcolor.h>
