@@ -22,6 +22,10 @@
 */
 // layout management added 1998/04/19 by Mario Weilguni <mweilguni@kde.org>
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include "xlock.h"		/* in xlockmore distribution */
 
 
