@@ -24,6 +24,9 @@
  * aggreed to distribute their code under the terms of the GPL
  */
 
+/* define _GNU_SOURCE to get prototypes for getpt() and ptsname() */
+#define _GNU_SOURCE
+
 #include <stdarg.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
@@ -41,7 +44,7 @@
 #include "debug.h"
 
 #if defined (_HPUX_SOURCE)
-#define _TERMIOS_INCLUDED
+/* #define _TERMIOS_INCLUDED */
 #include <bsdtty.h>
 #endif
 /* sgis have /dev/ptmx [bmg] */
@@ -77,8 +80,10 @@
 #endif
 
 #ifdef HAVE_TERMIO_H
+#ifndef HAVE_TERMIOS_H  /* Prefer new POSIX termios.h */
 #include <termio.h>
 #endif
+#endif
 
 #ifdef HAVE_TERMIOS_H
 #include <termios.h>
@@ -94,6 +99,10 @@
 #include "screen.h"
 #include "xsetup.h"
 
+#if !defined(HAVE_SETEUID)
+#define seteuid(_eu) setresuid(-1, _eu, -1)
+#endif
+
 #if defined(VDISCRD) && !defined(VDISCARD)
 #define	VDISCARD	VDISCRD
 #endif
@@ -165,12 +174,16 @@ static int x_fd;	/* file descriptor of the X server connection */
 static int fd_width;	/* width of file descriptors being used */
 static int app_cur_keys = 0;/* flag to set cursor keys in application mode */
 static int app_kp_keys = 0; /* flag to set application keypad keys */
+#if __GLIBC__ - 0 == 2 && __GLIBC_MINOR__ >= 1
+char *ttynam;
+#else
 #ifndef SVR4
-static char ptynam[] = "/dev/ptyxx";
-char ttynam[] = "/dev/ttyxx";
+static char ptynam[25] = "/dev/ptyxx";
+char ttynam[25] = "/dev/ttyxx";
 #else
 char *ttynam, *ptsname();
 #endif
+#endif
 static Atom wm_del_win;
 extern WindowInfo MyWinInfo;
 
@@ -332,8 +345,9 @@ static void catch_sig(int sig)
  * Command is ignored as parameter. Matthias */ 
  int run_command(unsigned char *command,unsigned char **argv)
 {
-  int ptyfd;
+  int ptyfd=-1;
   int uid, gid;
+  int grantpty=1;
   unsigned char *s3, *s4;
   int i;
   int width, height;
@@ -345,46 +359,101 @@ static void catch_sig(int sig)
   /*  First find a master pty that we can open.  
    */
 
-#ifdef SVR4
-  ptyfd = open("/dev/ptmx",O_RDWR);
-  if (ptyfd < 0) 
+#if __GLIBC__ - 0 == 2 && __GLIBC_MINOR__ >= 1
+  ptyfd = getpt();
+  if (ptyfd < 0)
     {
       error("Can't open a pseudo teletype");
       return(-1);
     }
-  grantpt(ptyfd);
-  unlockpt(ptyfd);
+  if (grantpt(ptyfd) < 0 || unlockpt(ptyfd) < 0)
+    {
+      close(ptyfd);
+      error("Can't grantpt/unlockpt a pseudo teletype");
+      return(-1);
+    }
+  ttynam = ptsname(ptyfd);
+  if (ttynam == 0)
+    {
+      close(ptyfd);
+      error("Pseudo teletype has no name");
+      return(-1);
+    }
   fcntl(ptyfd,F_SETFL,O_NDELAY);
-  ttynam=ptsname(ptyfd);
+  grantpty = 0;
 #else
-  ptyfd = -1;
-  for (s3 = ptyc3; *s3 != 0; s3++) 
-    {
-      for (s4 = ptyc4; *s4 != 0; s4++) 
+#ifdef TIOCGPTN
+  strcpy(ptynam,"/dev/ptmx");
+  strcpy(ttynam,"/dev/pts/");
+  ptyfd = open(ptynam,O_RDWR);
+  if (ptyfd >= 0) // got the master pty
+  { int ptyno;
+    if (ioctl(ptyfd, TIOCGPTN, &ptyno) == 0)
+    { struct stat sbuf;
+      sprintf(ttynam,"/dev/pts/%d",ptyno);
+      if (stat(ttynam,&sbuf) == 0 && S_ISCHR(sbuf.st_mode))
+	grantpty = 0;
+      else
 	{
-	  ptynam[8] = ttynam[8] = *s3;
-	  ptynam[9] = ttynam[9] = *s4;
-	  if ((ptyfd = open(ptynam,O_RDWR)) >= 0) 
-	    {
-	      if (geteuid() == 0 || access(ttynam,R_OK|W_OK) == 0)
-		break;
-	      else 
-		{
-		  close(ptyfd);
-		  ptyfd = -1;
-		}
-	    }
+	  close(ptyfd);
+	  ptyfd = -1;
 	}
-      if (ptyfd >= 0)
-	break;
     }
-  if (ptyfd < 0) 
+    else
     {
-      error("Can't open a pseudo teletype");
-      return(-1);
+      close(ptyfd);
+      ptyfd = -1;
     }
-  fcntl(ptyfd,F_SETFL,O_NDELAY);
+  }     
+#endif
+
+if (ptyfd < 0)
+  {
+#ifdef SVR4
+    ptyfd = open("/dev/ptmx",O_RDWR);
+    if (ptyfd < 0) 
+      {
+	error("Can't open a pseudo teletype");
+	return(-1);
+      }
+    grantpt(ptyfd);
+    unlockpt(ptyfd);
+    fcntl(ptyfd,F_SETFL,O_NDELAY);
+    ttynam=ptsname(ptyfd);
+#else
+    strcpy(ptynam, "/dev/ptyxx");
+    strcpy(ttynam, "/dev/ttyxx");
+    ptyfd = -1;
+    for (s3 = ptyc3; *s3 != 0; s3++) 
+      {
+	for (s4 = ptyc4; *s4 != 0; s4++) 
+	  {
+	    ptynam[8] = ttynam[8] = *s3;
+	    ptynam[9] = ttynam[9] = *s4;
+	    if ((ptyfd = open(ptynam,O_RDWR)) >= 0) 
+	      {
+		if (geteuid() == 0 || access(ttynam,R_OK|W_OK) == 0)
+		  break;
+		else 
+		  {
+		    close(ptyfd);
+		    ptyfd = -1;
+		  }
+	      }
+	  }
+	if (ptyfd >= 0)
+	  break;
+      }
+    if (ptyfd < 0) 
+      {
+	error("Can't open a pseudo teletype");
+	return(-1);
+      }
+    fcntl(ptyfd,F_SETFL,O_NDELAY);
 #endif  
+  }
+#endif /* GLIBC */
+
   for (i = 1; i <= 15; i++)
     signal(i,catch_sig);
   signal(SIGCHLD,catch_child);
@@ -419,8 +488,19 @@ static void catch_sig(int sig)
 	gid = gr->gr_gid;
       else
 	gid = -1;
-      fchown(ttyfd,uid,gid);
-      fchmod(ttyfd,0600);
+
+      if (grantpty)
+	{
+	  // regain root privileges
+	  seteuid(0);
+	  
+	  fchown(ttyfd,uid,gid);
+	  fchmod(ttyfd,0600);
+	  
+	  // drop root privileges again
+	  seteuid(getuid());
+	}
+
 #endif
 #ifdef TIOCCONS
      if (console) 
@@ -1790,7 +1870,7 @@ void process_csi_sequence()
 void process_xterm_sequence()
 {
   int c,n,i,arg;
-  unsigned char string[STRING_MAX];
+  unsigned char string[STRING_MAX+1];
 
   c = get_com_char(0);
   n = 0;
@@ -1803,7 +1883,7 @@ void process_xterm_sequence()
 
   c = get_com_char(0);
   i = 0;
-  while ( c != 7 && i < STRING_MAX)
+  while ( c != 7 && c != GCC_NULL && i < STRING_MAX)
     {
       if (c >= ' ')
 	string[i++] = c;
