@@ -4,6 +4,8 @@
  * Copyright (C) 1997 Martin Jones
  *               1998 Matej Koss
  *
+ * $Id: bg.cpp,v 1.33.2.9 1999/09/04 15:04:00 dmuell Exp $
+ *
  *
  */
 
@@ -11,6 +13,8 @@
 
 #include <stdlib.h>
 
+#include <X11/Xatom.h>
+
 #include <qimage.h>
 #include <qfile.h>
 #include <qpainter.h>
@@ -93,90 +97,83 @@ void KBackground::readSettings( int num, bool one, int onedesk )
   QString tmpf;
   ksprintf( &tmpf, "/desktop%drc", num);
 
-  bool first_time = false;
-
-  QFileInfo fi( KApplication::localconfigdir() + tmpf );
-  if ( ! fi.exists() ){
-    tmpf = "/kcmdisplayrc";
-    first_time = true;
-  }
-
   KConfig config(KApplication::kde_configdir() + tmpf,
 		  KApplication::localconfigdir() + tmpf);
 
-  if ( !first_time ) {
-    config.setGroup( "Common" );
-    randomMode = config.readBoolEntry( "RandomMode", DEFAULT_ENABLE_RANDOM_MODE );
+  config.setGroup( "Common" );
+  randomMode = config.readBoolEntry( "RandomMode", DEFAULT_ENABLE_RANDOM_MODE );
 
-    if ( randomMode ) {
-      if ( !timerRandom ) {
-	timerRandom = new QTimer( this );
-	connect(timerRandom, SIGNAL(timeout()), SLOT(randomize()));
-      }
-      timerRandom->stop();
-      timerRandom->start( config.readNumEntry( "Timer",
-                                              DEFAULT_RANDOM_TIMER ) * 1000 );
-      int count = config.readNumEntry( "Count", DEFAULT_RANDOM_COUNT );
-      bool inorder = config.readBoolEntry( "InOrder", DEFAULT_RANDOM_IN_ORDER);
-      useDir = config.readBoolEntry( "UseDir", DEFAULT_RANDOM_USE_DIR );
+  if ( randomMode ) {
+    if ( !timerRandom ) {
+      timerRandom = new QTimer( this );
+      connect(timerRandom, SIGNAL(timeout()), SLOT(randomize()));
+    }
+    timerRandom->stop();
+    timerRandom->start( config.readNumEntry( "Timer",
+                                            DEFAULT_RANDOM_TIMER ) * 1000 );
+    int count = config.readNumEntry( "Count", DEFAULT_RANDOM_COUNT );
+    bool inorder = config.readBoolEntry( "InOrder", DEFAULT_RANDOM_IN_ORDER);
+    useDir = config.readBoolEntry( "UseDir", DEFAULT_RANDOM_USE_DIR );
 
-      if ( useDir ) {
-	QString tmpd = config.readEntry( "Directory", KApplication::kde_wallpaperdir());
-	QDir d( tmpd, "*", QDir::Name, QDir::Readable | QDir::Files );
+    if ( useDir ) {
+      QString tmpd = config.readEntry( "Directory", KApplication::kde_wallpaperdir());
+      QDir d( tmpd, "*", QDir::Name, QDir::Readable | QDir::Files );
 
-	QStrList *list = (QStrList *)d.entryList();
+      QStrList *list = (QStrList *)d.entryList();
 
-	count = list->count();
-	if ( inorder ) {
-	  randomDesk = config.readNumEntry( "Item", DEFAULT_DESKTOP );
-	  randomDesk++;
-	  if ( randomDesk >= count ) randomDesk = DEFAULT_DESKTOP;
-	} else if ( count > 0 )
-	  randomDesk = random() % count;
-
-	config.writeEntry( "Item", randomDesk );
-	config.sync();
-
-	color1 = QColor(DEFAULT_COLOR_1);
-	gfMode = DEFAULT_COLOR_MODE;
-	orMode = DEFAULT_ORIENTATION_MODE;
-	wpMode = DEFAULT_WALLPAPER_MODE;
-	bUseWallpaper = true;
-
-	wallpaper = d.absPath() + "/" + list->at( randomDesk );
-	name.sprintf( "%s_%d_%d_%d#%02x%02x%02x#%02x%02x%02x#", wallpaper.data(),
-		      wpMode, gfMode, orMode, color1.red(), color1.green(),
-		      color1.blue(), color1.red(), color2.green(), color2.blue());
-
-	hasPm = true;
-
-	// this is mainly for kpager, so that we can at anytime find out how desktop
-	//          really looks
-	config.writeEntry( "Item", randomDesk );
-	config.sync();
-
-	return;
-      }
-      else if ( inorder ) {
-	randomDesk = config.readNumEntry( "Item", DEFAULT_DESKTOP );
+      count = list->count();
+      if ( inorder ) {
+        randomDesk = config.readNumEntry( "Item", DEFAULT_DESKTOP );
 	randomDesk++;
 	if ( randomDesk >= count ) randomDesk = DEFAULT_DESKTOP;
+      } else if ( count > 0 ) {
+        randomDesk = random() % count;
       }
-      else if ( count > 0 )
-	randomDesk = random() % count;
 
+      color1 = QColor(DEFAULT_COLOR_1);
+      gfMode = DEFAULT_COLOR_MODE;
+      orMode = DEFAULT_ORIENTATION_MODE;
+      wpMode = DEFAULT_WALLPAPER_MODE;
+      bUseWallpaper = true;
+
+      wallpaper = d.absPath() + "/" + list->at( randomDesk );
+      name.sprintf( "%s_%d_%d_%d#%02x%02x%02x#%02x%02x%02x#", wallpaper.data(),
+                    wpMode, gfMode, orMode, color1.red(), color1.green(),
+		    color1.blue(), color2.red(), color2.green(), color2.blue());
+
+      hasPm = true;
+
+      // this is mainly for kpager, so that we can at anytime find out how desktop
+      //          really looks
+      config.writeEntry( "Item", randomDesk );
+      config.sync();
+
+      return;
     }
-    else
+    else if ( inorder ) {
+      randomDesk = config.readNumEntry( "Item", DEFAULT_DESKTOP );
+      randomDesk++;
+      if ( randomDesk >= count ) randomDesk = DEFAULT_DESKTOP;
+    }
+    else if ( count > 0 ) {
+      randomDesk = random() % count;
+    }
+    else {
       randomDesk = DEFAULT_DESKTOP;
-
-    // this is mainly for kpager, so that we can at anytime find out how desktop
-    //          really looks
-    config.writeEntry( "Item", randomDesk );
-    config.sync();
+    }
+  }
+  else {
+    if(timerRandom) {
+      delete timerRandom;
+      timerRandom = 0;
+    }
+    randomDesk = DEFAULT_DESKTOP;
   }
-  else
-    randomDesk = num + 1;
 
+  // this is mainly for kpager, so that we can at anytime find out how desktop
+  //          really looks
+  config.writeEntry( "Item", randomDesk );
+  config.sync();
 
   ksprintf( &tmpf, "Desktop%d", randomDesk);
   config.setGroup( tmpf );
@@ -233,14 +230,14 @@ void KBackground::readSettings( int num, bool one, int onedesk )
     wpMode = Scaled;
 
   wallpaper = DEFAULT_WALLPAPER;
-  bUseWallpaper = config.readBoolEntry( "UseWallpaper", DEFAULT_WALLPAPER_MODE );
+  bUseWallpaper = config.readBoolEntry( "UseWallpaper", DEFAULT_USE_WALLPAPER );
   if ( bUseWallpaper )
     wallpaper = config.readEntry( "Wallpaper", DEFAULT_WALLPAPER );
 
   name.detach();
   name.sprintf( "%s_%d_%d_%d#%02x%02x%02x#%02x%02x%02x#", wallpaper.data(),
 		wpMode, gfMode, orMode, color1.red(), color1.green(),
-		color1.blue(), color1.red(), color2.green(), color2.blue());
+		color1.blue(), color2.red(), color2.green(), color2.blue());
 
   QString tmp;
   for (int i = 0; i < 8; i++) {
@@ -320,8 +317,9 @@ void KBackground::apply()
   bgPixmap = QPixmapCache::find( name );
   if ( bgPixmap )
     {
-      debug( "Desktop background found in cache" );
+      //      debug( "Desktop background found in cache" );
       qApp->desktop()->setBackgroundPixmap( *bgPixmap );
+      setPixmapProperty( bgPixmap );
       bgPixmap = 0;
       applied = true;
       return;
@@ -342,7 +340,8 @@ void KBackground::apply()
        (wpMode == CentredWarp) || ( wpMode == CentredMaxpect) ) {
     if (bgPixmap)
       bgPixmap->resize(w, h);
-	
+
+    KPixmap pmDesktop;
     switch (gfMode) {
 
     case Gradient:
@@ -350,19 +349,15 @@ void KBackground::apply()
 	int numColors = 4;
 	if ( QColor::numBitPlanes() > 8 )
 	  numColors = 16;
-		
-	KPixmap pmDesktop;
-		
+
 	if ( orMode == Portrait ) {
 
 	  pmDesktop.resize( 20, QApplication::desktop()->height() );
 	  pmDesktop.gradientFill( color2, color1, true, numColors );
 
 	} else {
-
 	  pmDesktop.resize( QApplication::desktop()->width(), 20 );
 	  pmDesktop.gradientFill( color2, color1, false, numColors );
-		
 	}
 
 	delete bgPixmap;
@@ -371,8 +366,9 @@ void KBackground::apply()
 	if (! wpPixmap ) {
 
 	  qApp->desktop()->setBackgroundPixmap(pmDesktop);
+          setPixmapProperty( &pmDesktop );
 	  *bgPixmap = pmDesktop;
-		
+
 	} else {
 	  bgPixmap->resize(w, h);
 		
@@ -382,7 +378,7 @@ void KBackground::apply()
 		      pmDesktop.width(), h);
 	  } else {
 	    for (uint ph = 0; ph <= h; ph += pmDesktop.height()) {
-	      debug("land %d",ph);
+	      //	      debug("land %d",ph);
 	      bitBlt( bgPixmap, 0, ph, &pmDesktop, 0, 0,
 		      w, pmDesktop.height());
 	    }
@@ -430,6 +426,7 @@ void KBackground::apply()
 
 	if (! wpPixmap ) {
 	  qApp->desktop()->setBackgroundPixmap(tile);
+          setPixmapProperty( &pmDesktop );
 	  *bgPixmap = tile;
 	  applied = true;
 	} else {
@@ -716,11 +713,30 @@ void KBackground::timerEvent( QTimerEvent * )
   if ( !bgPixmap )
     return;
 
+  QPixmap pmDesktop;
   qApp->desktop()->setBackgroundPixmap( *bgPixmap );
+  setPixmapProperty( &pmDesktop );
   delete bgPixmap;
   bgPixmap = 0;
   applied = true;
 }
 
+void KBackground::setPixmapProperty( QPixmap *qP )
+{
+    Display *Xdisplay = kapp->getDisplay();
+    Window Xroot = kapp->desktop()->handle();
+    Pixmap p = qP->handle();
+    Atom prop_root = XInternAtom(Xdisplay, "_XROOTPMAP_ID", False);
+    Atom prop_esetroot = XInternAtom(Xdisplay, "ESETROOT_PMAP_ID", False);
+
+    if(prop_root == None || prop_esetroot == None) return;
+
+    XChangeProperty(Xdisplay, Xroot, prop_root, XA_PIXMAP, 32, PropModeReplace,
+                    (unsigned char *) &p, 1);
+    XChangeProperty(Xdisplay, Xroot, prop_esetroot, XA_PIXMAP, 32,
+                    PropModeReplace, (unsigned char *) &p, 1);
+}
+
+
 //----------------------------------------------------------------------------
 
